from op5helpers import *
import time

log("Using %s as api endpoint" % api_endpoint)

def test_contact_crud():
    crud = Crud('contact', 'special_test_contact')

    crud.post({
        'contact_name': 'special_test_contact',
        'alias': 'special_test_contact',
        'pager': '0123456789'})
    crud.get()

    crud.put({
        'contact_name': 'special_test_contact',
        'alias': 'special_test_contact',
        'email': 'foo@bar.com'})
    crud.get()
    crud.delete()
    crud.verify_gone()

def test_contactgroup_crud():
    contacts = ['a', 'b', 'c', 'd', 'e']
    gencntct = lambda(name): {'contact_name': name, 'alias': name}
    for cntct in contacts:
        crud = Crud('contact', cntct)
        crud.post(gencntct(cntct))
    Crud('contactgroup', 'a_new_renamed_group')
    crud = Crud('contactgroup', 'a_new_group')
    data = {
        'contactgroup_name': 'a_new_group',
        'alias': 'a_new_group',
        'members': ['monitor', 'a', 'b', 'c', 'd']}
    crud.post(data)
    crud.get()

    old_name = crud.name
    new_name = 'a_new_renamed_group'
    data['contactgroup_name'] = new_name
    data['members'] = ['e']

    crud.put(data)
    crud.name = new_name
    crud.get()

    crud.name = old_name
    crud.verify_gone()
    crud.name = new_name

    crud.delete()
    crud.verify_gone()
    for cntct in contacts:
        crud = Crud('contact', cntct)

def test_host_crud():
    crud = Crud('host', 'special_test_host')
    crud.post({
        'host_name': 'special_test_host',
        'address': 'original_address',
        'hostgroups': ['unix-servers', 'network'],
        'active_checks_enabled': True,
        'stalking_options': ['d', 'u', 'o'],
        '_CUSTOM': "it's here"
    })
    crud.get()

    crud.put({
        'host_name': 'special_test_host',
        'address': 'changed_address',
        'hostgroups': ['win-servers'],
        'notes': "let's have a note"})
    crud.get()

    crud.patch({'address': 'third_address'})
    crud.get()

    out = crud.put({
        'host_name': 'special_test_host',
        'address': 'original_address',
        'hostgroups': ['unix-servers', 'network']})
    eq_(out['notes'], u'')
    crud.get()

    crud.delete()
    crud.verify_gone()

def test_hostgroup_crud():
    crud = Crud('hostgroup', 'test_group')
    crud.post({
        'hostgroup_name': 'test_group',
        'alias': 'a_new_group',
        'members': ['monitor', 'linux-server1']})
    crud.get()
    crud.patch({'members': ['linux-server1']})
    crud.get()

    crud.delete()
    crud.verify_gone()

def test_host_service_crud():
    Crud('service', 'monitor;renamed_test_service')
    crud = Crud('service', 'monitor;test_service')
    crud.post({
        'host_name': 'monitor',
        'service_description': 'test_service',
        'check_command': 'check-host-alive',
        'contacts': ['monitor']})
    crud.get()
    crud.patch({'service_description': 'renamed_test_service'})
    crud.name = 'monitor;test_service'
    crud.verify_gone()
    crud.name = 'monitor;renamed_test_service'
    crud.get()
    out = crud.patch({'check_command': 'check-invalid'}, cmp=False, expected_code=400)
    assert_in('error', out)
    assert_in("it doesn't exist", out['full_error'])
    crud.delete()
    crud.verify_gone()

def test_hostgroup_service_crud():
    Crud('service', 'unix-servers;renamed_test_service')
    crud = Crud('service', 'unix-servers;test_service', get_params={'parent_type': 'hostgroup'})
    crud.post({
        'hostgroup_name': 'unix-servers',
        'service_description': 'test_service',
        'check_command': 'check-host-alive',
        'contacts': ['monitor']})
    crud.get()
    crud.patch({'service_description': 'renamed_test_service'})
    crud.name = 'unix-servers;test_service'
    crud.verify_gone()
    crud.name = 'unix-servers;renamed_test_service'
    crud.get()
    out = crud.patch({'check_command': 'check-invalid'}, cmp=False, expected_code=400)
    assert_in('error', out)
    assert_in("it doesn't exist", out['full_error'])
    crud.delete()
    crud.verify_gone()

def test_servicegroup_crud():
    crud = Crud('servicegroup', 'test_group')
    crud.post({
        'servicegroup_name': 'test_group',
        'alias': 'a_new_group',
        'members': ['linux-server1;PING', 'router1;PING']})
    crud.get()
    crud.patch({'members': ['linux-server1;PING']})
    crud.get()

    crud.delete()
    crud.verify_gone()

def test_timeperiod_crud():
    '''Create all the "weird" timeperiods from nagios' docs'''
    crud = Crud('timeperiod', 'misc-tp-tests')
    crud.post({
        'timeperiod_name': 'misc-tp-tests',
        'alias': 'Misc Date Ranges',
        '2007-01-01 - 2008-02-01': '00:00-24:00',
        'april 10 - may 15': '00:00-24:00',
        'day 1 - 15': '00:00-24:00',
        'day 20 - -1': '00:00-24:00',
        'july 10 - 15': '00:00-24:00',
        'monday 3 - thursday 4': '00:00-24:00',
        'tuesday 1 april - friday 2 may': '00:00-24:00',
    })
    crud.get()

    out = crud.put({
        'timeperiod_name': 'misc-tp-tests',
        'alias': 'Misc Single Days',
        '1999-01-28': '00:00-24:00',
        'day 2': '00:00-24:00',
        'february -1': '00:00-24:00',
        'february 10': '00:00-24:00',
        'friday -2': '00:00-24:00',
        'monday 3': '00:00-24:00',
        'thursday -1 november': '00:00-24:00',
    })
    assert_not_in('april 10 - may 15', out)
    out = crud.get()
    assert_not_in('april 10 - may 15', out)

    crud.put({
        'timeperiod_name': 'misc-tp-tests',
        'alias': 'Misc Skip Ranges',
        '2007-01-01 - 2008-02-01 / 3': '00:00-24:00',
        '2008-04-01 / 7': '00:00-24:00',
        'day 1 - 15 / 5': '00:00-24:00',
        'july 10 - 15 / 2': '00:00-24:00',
        'monday 3 - thursday 4 / 2': '00:00-24:00',
        'tuesday 1 april - friday 2 may / 6': '00:00-24:00',
    })
    crud.get()

    crud.delete()
    crud.verify_gone()

def test_command_crud():
    crud = Crud('command', 'new_command')
    crud.post({
        'command_name': 'new_command',
        'command_line': '/bin/true',
    })
    crud.get()
    crud.patch({'command_line': '/bin/false'})
    crud.get()

    crud.delete()
    crud.verify_gone()

def test_user_crud():
    crud = Crud('user', 'a_test_user')

    data = {
        'Username': 'a_test_user',
        'Password': 's3kr1t',
        'Access Levels': ['authorized_for_all_hosts']}

    out = crud.post(data, cmp=False)
    data.pop('Password')
    data.pop('Access Levels')
    data['groups'] = ['user_a_test_user']
    nested_equal(data, out)
    crud.get()

    data = {
        'Username': 'a_test_user',
        'Password': 's3kr1ter',
        'Access Levels': []}
    out = crud.put(data, cmp=False)
    data.pop('Password')
    nested_equal(data, out)
    crud.get()
    crud.delete()
    crud.verify_gone()
    crud = Crud('usergroup', 'user_a_test_user')

def test_usergroup_crud():
    crud = Crud('usergroup', 'super_group')

    data = {
        'name': 'super_group',
        'access_rights': [
            'service_add_delete',
            'hostdependency_add_delete'
        ]}

    crud.post(data)
    crud.get()
    crud.delete()
    crud.verify_gone()

def test_service_in_host():
    crud = Crud('host', 'testhost')
    out = crud.post({
            'host_name': 'testhost',
            'services': [
                {'service_description': 'service1',
                 'check_command': 'check-host-alive'},
                {'service_description': 'service2',
                 'check_command': 'check-host-alive'}]})
    for svc in out['services']:
        subcrud = Crud('service', 'testhost;'+svc['service_description'], clean=False)
        subcrud.get()
    crud.get()
    crud.delete()
    crud.verify_gone()
    for svc in out['services']:
        subcrud = Crud('service', 'testhost;'+svc['service_description'], clean=False)
        subcrud.verify_gone()

def test_old_obsess_names():
    crud = Crud('host', 'testhost')
    out = crud.post({
            'host_name': 'testhost',
            'obsess_over_host': True,
            'services': [
                {'service_description': 'service',
                 'check_command': 'check-host-alive',
                 'obsess_over_service': True
                 }]})
    assert_in('obsess', out)
    ok_(out['obsess'])
    assert_in('obsess_over_host', out)
    ok_(out['obsess_over_host'])
    for svc in out['services']:
        subcrud = Crud('service', 'testhost;'+svc['service_description'], clean=False)
        out = subcrud.get()
        assert_in('obsess', out)
        ok_(out['obsess'])
        assert_in('obsess_over_service', out)
        ok_(out['obsess_over_service'])
    crud.get()
    crud.delete()
    crud.verify_gone()

def test_new_obsess_names():
    crud = Crud('host', 'testhost')
    out = crud.post({
            'host_name': 'testhost',
            'obsess': True,
            'services': [
                {'service_description': 'service',
                 'check_command': 'check-host-alive',
                 'obsess': True
                 }]})
    assert_in('obsess', out, 'new name present')
    ok_(out['obsess'], 'new name is true')
    assert_in('obsess_over_host', out, 'old name present')
    ok_(out['obsess_over_host'], 'old name is true')
    for svc in out['services']:
        subcrud = Crud('service', 'testhost;'+svc['service_description'], clean=False)
        out = subcrud.get()
        assert_in('obsess', out, 'new name present for service')
        ok_(out['obsess'], 'new name is true for service')
        assert_in('obsess_over_service', out, 'old name present for service')
        ok_(out['obsess_over_service'], 'old name is true for service')
    crud.get()
    crud.delete()
    crud.verify_gone()

def test_service_in_hostgroup():
    crud = Crud('hostgroup', 'testgroup')
    crud.post({
            'hostgroup_name': 'testgroup',
            'alias': 'testgroup',
            'members': ['monitor'],
            'services': [
                {'service_description': 'service1',
                 'check_command': 'check-host-alive'},
                {'service_description': 'service2',
                 'check_command': 'check-host-alive'}]})
    out = crud.get()
    for svc in out['services']:
        subcrud = Crud('service', 'testgroup;'+svc['service_description'], clean=False, get_params = {'parent_type': 'hostgroup'})
        subcrud.get()
    crud.get()
    crud.delete()
    crud.verify_gone()
    for svc in out['services']:
        subcrud = Crud('service', 'testgroup;'+svc['service_description'], clean=False, get_params = {'parent_type': 'hostgroup'})
        subcrud.verify_gone()

def test_hosttemplate_crud():
    crud = Crud('host_template', 'new-template')
    crud.post({
        'name': 'new-template',
        'check_interval': 3})
    crud.get()
    crud.patch({'check_interval': 5})
    crud.get()
    crud.delete()
    crud.verify_gone()

def test_servicetemplate_crud():
    crud = Crud('service_template', 'other-new-template')
    crud.post({
        'name': 'other-new-template',
        'display_name': 'foo'})
    crud.get()
    crud.put({
        'name': 'other-new-template',
        'display_name': 'foo1'})
    crud.get()
    crud.delete()
    crud.verify_gone()

def test_servicetemplate_members():
    """ bug #6415 """
    template = u'dalton'
    check = u'check_dummy'
    host = u'jolly'
    service = u'ratata'
    contacts = [u'billy_the_kid', u'jesse_james']
    contact_groups = [u'willy', u'wonka']

    Change.delete()

    for contact in contacts:
        con = Crud('contact', contact)
        con.post({
            'contact_name': contact,
            'alias': contact,
            'pager': '0123456789'
        })
        con.get()

    for contact_group in contact_groups:
        cg = Crud('contactgroup', contact_group)
        cg.post({
            'contactgroup_name': contact_group,
            'alias': 'a_new_group',
            'members': contacts})
        cg.get()

    h = Crud('host', host)
    h.post({
        'host_name': host,
        'address': 'penetentiary',
        'active_checks_enabled': True,
        'stalking_options': ['d', 'u', 'o'],
    })

    t = Crud('service_template', template)
    t.post({
        'name': template,
        'display_name': template,
        'check_command': check,
        'notification_period': '24x7',
        'contact_groups': contact_groups,
        'contacts': contacts,
    })
    result = t.get()

    s = Crud('service', host + ";" + service)
    s.post({
        'host_name': host,
        'service_description': service,
        'template': template
    })
    result = s.get()

    # start cleanup
    s.delete()
    s.verify_gone()

    t.delete()
    t.verify_gone()

    h.delete()
    h.verify_gone()

    for contact in contacts:
        con = Crud('contact', contact, clean=False)
        con.delete()
        con.verify_gone()

    for contact_group in contact_groups:
        cg = Crud('contactgroup', contact_group, clean=False)
        cg.delete()
        cg.verify_gone()
    # end cleanup

    eq_(result['contacts'], contacts)
    eq_(result['check_command'], check)
    eq_(result['contact_groups'], contact_groups)

def test_contacttemplate_crud():
    crud = Crud('contact_template', 'another-new-template')
    crud.post({
        'name': 'another-new-template',
        'pager': '0123456789'})
    crud.get()
    crud.put({
        'name': 'another-new-template',
        'email': 'foo@bar.com'})
    crud.get()
    crud.delete()
    crud.verify_gone()

def test_role_removed_message():
    resp = requests.post(api_endpoint + 'api/config/role/', auth=('monitor', 'monitor'), verify=False, data={'role_name': 'api_test_role',
        'host': ['no-access']})
    print resp.text
    ok_("no longer exist" in resp.text)
    eq_(resp.status_code, 404)

def test_hostdep_in_host():
    crud = Crud('host', 'new_host')
    out = crud.post({
        'host_name': 'new_host',
        'hostdependencys': [{
            'host_name': 'monitor'
            }]})
    assert_not_in('resource', out['hostdependencys'][0].keys())
    out = crud.get()
    assert_not_in('resource', out['hostdependencys'][0].keys())
    crud.delete()

def test_hostescalations_in_host():
    crud = Crud('host', 'new_host')
    crud.post({
        'host_name': 'new_host',
        'hostescalations': [{
            'contacts': ['monitor'],
            'first_notification': 3,
            'last_notification': 3,
            'notification_interval': 3,
            }]})

def test_servicedep_in_service():
    crud = Crud('service', 'monitor;new_service')
    crud.post({
        'host_name': 'monitor',
        'service_description': 'new_service',
        'check_command': 'check-host-alive',
        'servicedependencys': [{
            'service': 'linux-server1;PING'}]})
    crud.get()
    crud.delete()
    crud.verify_gone()

def test_serviceescalation_in_service():
    crud = Crud('service', 'monitor;new_service')
    crud.post({
        'host_name': 'monitor',
        'service_description': 'new_service',
        'check_command': 'check-host-alive',
        'serviceescalations': [{
            'contacts': ['monitor']
            }]})
    crud.get()
    crud.delete()
    crud.verify_gone()

def test_servicesubobjs_in_host():
    crud = Crud('host', 'new_host')
    out = crud.post({
        'host_name': 'new_host',
        'services': [{
            'service_description': 'first_service',
            'check_command': 'check-host-alive',
            'servicedependencys': [{
                'service': 'linux-server1;PING'}],
            'serviceescalations': [{
                'contacts': ['monitor']}]}]})
    crud.get()
    for svc in out['services']:
        subcrud = Crud('service', 'new_host;'+svc['service_description'], clean=False)
        subcrud.get()
    crud.delete()
    crud.verify_gone()
    for svc in out['services']:
        subcrud = Crud('service', 'testgroup;'+svc['service_description'], clean=False)
        subcrud.verify_gone()

def test_invalid_object_type():
    r = requests.get(api_endpoint + 'api/config/marlboro-light?format=json',
            auth=('monitor', 'monitor'),
            verify=False)
    out = dejson(r.text)
    eq_(r.status_code, 404)
    eq_(r.headers['Content-Type'], 'application/json; charset=utf-8')

def test_illegal_class_type():
    r = requests.get(api_endpoint + 'api/config/gui?format=json',
            auth=('monitor', 'monitor'),
            verify=False)
    out = dejson(r.text)
    eq_(r.status_code, 404)
    eq_(r.headers['Content-Type'], 'application/json; charset=utf-8')

def test_post_same_object_twice():
    crud = Crud('host', 'new_host')
    crud.post({
        'host_name': 'new_host'})
    crud.get()
    out = crud.post({
        'host_name': 'new_host'},
        cmp=False, expected_code=409)
    eq_(out['error'], 'Object already exists')
    crud.get()
    crud.delete()
    crud.verify_gone()

def test_put_to_create():
    crud = Crud('host', 'new_host')
    crud.post({
        'host_name': 'new_host'})
    crud.get()
    crud.delete()
    crud.verify_gone()

def test_put_to_replace_sub():
    '''PUT:ing an existing object with existing sub-objects should replace them
    iff the sub-object-tag is present'''
    crud = Crud('host', 'new_host')
    subcrud = Crud('service', 'new_host;a_service')
    crud.post({
        'host_name': 'new_host',
        'services': [{
            'service_description': 'a_service',
            'check_command': 'check-host-alive'}]})
    crud.get()
    subcrud.get()
    crud.put({
        'host_name': 'new_host',
        'notes': 'change'})
    crud.get()
    subcrud.get()
    crud.patch({'action_url': 'http://foo.bar'})
    crud.get()
    subcrud.get()
    out = crud.put({
        'host_name': 'new_host',
        'services': []}, cmp=False)
    exp = crud.data
    del exp['services']
    nested_equal(exp, out)
    subcrud.get(cmp=False, expected_code=404)
    crud.put({
        'host_name': 'new_host',
        'hostdependencys': [{
            'host_name': 'monitor'}]})
    crud.put({
        'host_name': 'new_host',
        'hostdependencys': [{
            'host_name': 'linux-server1'}]})

def test_copy_with_sub():
    Crud('host', 'copied_host')
    crud = Crud('host', 'base_host')
    crud.post({
        'host_name': 'base_host',
        'address': 'localhost',
        'services': [{
            'service_description': 'a service',
            'check_command': 'check_dummy'
    }]})
    loaded = crud.get()
    loaded['host_name'] = 'copied_host'
    crud.post(loaded)
    loaded['host_name'] = 'base_host'
    crud.get()
    crud.name = 'copied_host'
    loaded['host_name'] = 'copied_host'
    crud.get()
    crud.delete()
    crud.name = 'base_host'
    crud.delete()

def test_auth():
    Crud('usergroup', 'user_test_limited')
    role = Crud('usergroup', 'restricted_group')
    role.post({
        'name': 'restricted_group',
        'access_rights': ['host_view_contact', 'host_edit_contact', 'api_config']
    })
    # Delete any pending
    r = Change.delete(False)
    dejson(r.text)
    r = requests.delete(api_endpoint + 'api/config/user/test_limited',
            auth=('monitor', 'monitor'),
            verify=False)
    r = requests.delete(api_endpoint + 'api/config/contact/test_limited',
            auth=('monitor', 'monitor'),
            verify=False)
    # Create new user, using old-school properties, with contact, only authed-by-contact
    r = requests.post(api_endpoint + 'api/config/user?format=json',
            auth=('monitor', 'monitor'),
            verify=False,
            headers={'content-type': 'application/json'},
            data=json.dumps({'Username': 'test_limited',
                'Password': 'test_limited',
                'Access Levels': ['authorized_for_system_information',
                    'authorized_for_configuration_information'],
                'Role': 'restricted_group'}))
    dejson(r.text)
    eq_(r.status_code, 201)
    r = requests.post(api_endpoint + 'api/config/contact?format=json',
            auth=('monitor', 'monitor'),
            verify=False,
            headers={'content-type': 'application/json'},
            data=json.dumps({'contact_name': 'test_limited',
                'alias': 'test_limited'}))
    dejson(r.text)
    eq_(r.status_code, 201)
    # Make new user contact for host
    r = requests.patch(api_endpoint + 'api/config/host/monitor?format=json',
            auth=('monitor', 'monitor'),
            verify=False,
            headers={'content-type': 'application/json'},
            data=json.dumps({'contacts': ['test_limited']}))
    eq_(r.status_code, 200)
    # Grab any hosts for user - nothing's been saved, so empty
    r = requests.get(api_endpoint + 'api/config/host?format=json',
            verify=False,
            auth=('test_limited', 'test_limited'))
    out = dejson(r.text)
    eq_(r.status_code, 200)
    eq_(out, [], 'There should not be any hosts in output yet: '+r.text)
    # Save config changes - this gives limited user access
    r = Change.post()
    dejson(r.text)
    # Verify said access
    r = requests.get(api_endpoint + 'api/config/host?format=json',
            verify=False,
            auth=('test_limited', 'test_limited'))
    out = dejson(r.text)
    eq_(r.status_code, 200)
    eq_(len(out), 1, "There should be one host in output: %s" % out)
    r = requests.get(api_endpoint + 'api/config/host/monitor?format=json',
            verify=False,
            auth=('test_limited', 'test_limited'))
    out = dejson(r.text)
    eq_(r.status_code, 200, r.text)
    nested_equal({'host_name': 'monitor'}, out)
    # Verify other stuff still fails
    r = requests.get(api_endpoint + 'api/config/host/linux-server1?format=json',
            verify=False,
            auth=('test_limited', 'test_limited'))
    dejson(r.text)
    eq_(r.status_code, 403)
    r = requests.patch(api_endpoint + 'api/config/host/linux-server1?format=json',
            verify=False,
            auth=('test_limited', 'test_limited'),
            headers={'content-type': 'application/json'},
            data=json.dumps({'alias': 'linux-server2'}))
    dejson(r.text)
    eq_(r.status_code, 403)
    r = requests.put(api_endpoint + 'api/config/host/linux-server1?format=json',
            verify=False,
            auth=('test_limited', 'test_limited'),
            headers={'content-type': 'application/json'},
            data=json.dumps({'alias': 'linux-server2'}))
    dejson(r.text)
    eq_(r.status_code, 403)
    # And clean up
    role.delete()
    Crud('user', 'test_limited')
    Crud('contact', 'test_limited')
    Crud('usergroup', 'user_test_limited')
    Change.post()

def test_push_invalid():
    crud = Crud('contact', 'invalid_contact')
    crud.verify_gone()
    crud = Crud('host', 'a_new_host')
    crud.verify_gone()
    r = requests.post(api_endpoint + 'api/config/host',
        auth=('monitor', 'monitor'),
        verify=False,
        data=json.dumps({
            'host_name': 'a_new_host',
            'contacts': ['invalid_contact']}),
        headers={'content-type': 'application/json'})

    eq_(r.status_code, 400)
    out = dejson(r.text)
    assert_in('invalid_contact', out['full_error'])

def test_push_limited_rels():
    role = Crud('usergroup', 'restricted_group')
    role.post({
        'name': 'restricted_group',
        'access_rights': ['host_view_contact', 'host_edit_contact', 'api_config']
    })
    r = requests.delete(api_endpoint + 'api/config/user/test_limited',
            auth=('monitor', 'monitor'),
            verify=False)
    r = requests.delete(api_endpoint + 'api/config/contact/test_limited',
            auth=('monitor', 'monitor'),
            verify=False)
    r = requests.post(api_endpoint + 'api/config/user?format=json',
            auth=('monitor', 'monitor'),
            verify=False,
            headers={'content-type': 'application/json'},
            data=json.dumps({'Username': 'test_limited',
                'Password': 'test_limited',
                'Access Levels': ['authorized_for_system_information',
                    'authorized_for_configuration_information'],
                'Role': 'restricted_group'}))
    dejson(r.text)
    eq_(r.status_code, 201, "Output: "+r.text)
    r = requests.post(api_endpoint + 'api/config/contact?format=json',
            auth=('monitor', 'monitor'),
            verify=False,
            headers={'content-type': 'application/json'},
            data=json.dumps({'contact_name': 'test_limited',
                'alias': 'test_limited'}))
    dejson(r.text)
    eq_(r.status_code, 201, "Output: "+r.text)
    r = requests.patch(api_endpoint + 'api/config/host/monitor?format=json',
            auth=('monitor', 'monitor'),
            verify=False,
            headers={'content-type': 'application/json'},
            data=json.dumps({'contacts': ['test_limited']}))
    dejson(r.text)
    eq_(r.status_code, 200, "Output: "+r.text)
    r = requests.patch(api_endpoint + 'api/config/host/monitor?format=json',
            verify=False,
            auth=('test_limited', 'test_limited'),
            headers={'content-type': 'application/json'},
            data=json.dumps({'contacts': ['monitor']}))
    dejson(r.text)
    eq_(r.status_code, 403, "Output: "+r.text)
    requests.delete(api_endpoint + 'api/config/user/test_limited',
            auth=('monitor', 'monitor'),
            verify=False)
    requests.delete(api_endpoint + 'api/config/usergroup/user_test_limited',
            auth=('monitor', 'monitor'),
            verify=False)
    requests.delete(api_endpoint + 'api/config/contact/test_limited',
            auth=('monitor', 'monitor'),
            verify=False)

def test_service_rels_left_after_subrel_put():
    '''Given that a relation exists to a service, and it's PUT-updated via
    a PUT to it's master host, the relation still sticks around'''
    other_service = Crud('service', 'linux-server1;other_service')
    parent = Crud('host', 'parent_host')
    service_name = 'parent_host;test_svc'
    parent_data = {
        'host_name': 'parent_host',
        'services': [{
            'service_description': 'test_svc',
            'check_command': 'check_dummy'
    }]}
    parent.put(parent_data)
    other_service.put({
        'host_name': 'linux-server1',
        'service_description': 'other_service',
        'check_command': 'check_dummy',
        'servicedependencys': [{
            'service': service_name
    }]})
    parent.put(parent_data)
    other_service.get()
    other_service.delete()
    parent.delete()

def test_service_rels_gone_after_subrel_replace():
    '''Given that a relation exists to a service, and the service list is PUT
    replaced via a PUT to it's master host, the relation is gone'''
    other_service = Crud('service', 'linux-server1;other_service')
    parent = Crud('host', 'parent_host')
    service_name = 'parent_host;test_svc'
    parent_data = {
        'host_name': 'parent_host',
        'services': [{
            'service_description': 'test_svc',
            'check_command': 'check_dummy'
    }]}
    parent.put(parent_data)
    other_service.put({
        'host_name': 'linux-server1',
        'service_description': 'other_service',
        'check_command': 'check_dummy',
        'servicedependencys': [{
            'service': service_name
    }]})
    parent_data['services'] = [{'service_description': 'other_test_svc', 'check_command': 'check_dummy'}]
    parent.put(parent_data)
    del other_service.data['servicedependencys']
    out = other_service.get()
    assert_not_in('servicedependencys', out)
    other_service.delete()
    parent.delete()

def test_subobj_thats_not_array():
    parent = Crud('host', 'a_host')
    out = parent.put({
        'host_name': 'parent_host',
        'services': 'yes please'
    }, expected_code=400, cmp=False)
    assert_in("'services'", out['full_error'])
    assert_in('list', out['full_error'])
    out = parent.put({
        'host_name': 'parent_host',
        'services': {'yes please': "i'll take two"}
    }, expected_code=400, cmp=False)
    assert_in("'services'", out['full_error'])
    assert_in('list', out['full_error'])

def test_changelog():
    r = Change.delete()
    dejson(r.text)

    r = Change.get()
    assert_list_equal(dejson(r.text), [])
    crud = Crud('host', 'new_host')
    crud.post({
        'host_name': 'new_host'})
    r = Change.get()
    eq_('new_host', dejson(r.text)[0]['name'])
    r = Change.post()
    eq_('Changes saved', dejson(r.text))
    # TODO: Does it turn up in ninja?
    crud.delete()
    r = Change.post()
    eq_('Changes saved', dejson(r.text))

def test_xml_response_from_xml_request():
    requests.delete(api_endpoint + 'api/config/host/arnold', auth=('monitor', 'monitor'), verify=False)
    data = """<?xml version="1.0" encoding="utf-8"?>
<root>
    <param name="host_name">arnold</param>
    <param name="alias">schwarzenegger</param>
    <param name="address">195.249.40.160</param>
    <param name="flap_detection_options">
        <value>d</value>
        <value>o</value>
        <value>u</value>
    </param>
    <param name="notes_url">/</param>
    <param name="action_url">http://nowhere</param>
    <param name="icon_image">op5eye.png</param>
    <param name="icon_image_alt">A server</param>
    <param name="statusmap_image">op5eye.png</param>
    <param name="template">default-host-template</param>
    <param name="file_id">etc/hosts.cfg</param>
    <param name="hostgroups">
        <value>network</value>
        <value>unix-servers</value>
    </param>
</root>"""
    r = requests.post(api_endpoint + 'api/config/host?format=xml', data=data, headers={ 'content-type': 'application/xml' }, auth=('monitor', 'monitor'), verify=False)
    eq_(r.status_code, 201, """A host was not created when POSTing the following data:

%s

This was the response body: %s
    """ % (data, r.text))
    eq_(r.headers['Content-Type'], 'application/xml; charset=utf-8', 'Content type not xml, here is the body: %s' % r.text)
    tree = dexml(r.text)
    exp = dexml(data)
    etree_nested_equal(exp, tree)
    r = requests.delete(api_endpoint + 'api/config/host/arnold?format=xml', auth=('monitor', 'monitor'), verify=False)
    dexml(r.text)

def test_unparsable_xml():
    data = """<root><>&<P"""
    r = requests.post(api_endpoint + 'api/config/timeperiod?format=xml', data=data, auth=('monitor', 'monitor'), headers={'content-type': 'application/xml'}, verify=False)
    dexml(r.text)
    eq_(400, r.status_code)

def test_invalid_xml():
    data = """<root><alias>asdfhasd asdf</a_property_here></root>"""
    r = requests.post(api_endpoint + 'api/config/timeperiod?format=xml', data=data, auth=('monitor', 'monitor'), headers={'content-type': 'application/xml'}, verify=False)
    dexml(r.text)
    eq_(400, r.status_code)

def test_wrong_xml():
    object_name = 'wassa'
    crud = Crud('timeperiod', object_name)
    data = """<root><alias>asdfhasd asdf</alias></root>"""
    r = requests.put(api_endpoint + 'api/config/timeperiod/'+object_name+'?format=xml', data=data, auth=('monitor', 'monitor'), headers={'content-type': 'application/xml'}, verify=False)
    dexml(r.text)
    eq_(400, r.status_code)

def test_noname_xml():
    object_name = 'wassa'
    crud = Crud('timeperiod', object_name)
    data = """<root><param>asdf</param></root>"""
    r = requests.put(api_endpoint + 'api/config/timeperiod/'+object_name+'?format=xml', data=data, auth=('monitor', 'monitor'), headers={'content-type': 'application/xml'}, verify=False)
    dexml(r.text)
    eq_(400, r.status_code)

def test_custom_variables_starts_with_underscore():
    object_name = "bamse"
    crud = Crud("host", object_name)
    data = {
        "address": "127.0.0.1",
        "host_name": "bamse",
        "alias": "bamse",
        "_lucky": "luke"
        }
    crud.put(data)

    data["sweet_stuff"] = "honey"
    crud.post(data, False, 400)
    crud.delete()

def test_can_not_create_user_and_contact_with_one_request():
    object_name = "popeye"
    crud = Crud("contact", object_name)
    data = {
        "contact_name": object_name,
        "alias": object_name,
        "Username": object_name,
        "Password": "Olivia",
        "Access Levels": {
            "authorized_for_all_hosts": 1
            }
        }
    crud.put(data, False, 400)

def test_create_user_with_everything_old_style():
    crud = Crud("user", "test_superduperuser")
    group = Crud("usergroup", "user_test_superduperuser")
    crud.post({
        'username': 'test_superduperuser',
        'password': 'superdupersecure',
        'Access Levels': ['authorized_for_all_hosts', 'authorized_for_system_commands', 'authorized_for_configuration_information']}, cmp=False)
    out = group.get()
    ok_(out['access_rights'].count('access_rights'), 'Superuser access right set')
    ok_(out['access_rights'].count('pnp'), 'PNP access right set')
    # yeah, sounds good, I guess...
    crud.delete()
    group.delete()

def test_slash_in_service_description():
    crud = Crud("service", "linux-server1;Is there any disk in /opt/monitor/var")
    crud.post({
        'host_name': 'linux-server1',
        'service_description': 'Is there any disk in /opt/monitor/var',
        'check_command': 'check-host-alive',
    })
    crud.get()
    crud.delete()
    crud.verify_gone()

def test_changelog_diffs_between_zero_and_null():
    """ bug #6636
    Warning: if this test fails after a Change.post()'s been made, your config
    files are altered. That's what we get for doing acceptance tests.
    ack -a monstar /opt/monitor/etc should help you find the changes to delete
    """
    Change.delete()
    r = Change.get()
    eq_(len(r.json), 0, "Carte blanche is expected")

    template = Crud('contact_template', 'monstarrs')
    template.post({
        'name': 'monstarrs',
        'pager': '0123456789',
        'service_notifications_enabled': True
    })

    contact = Crud('contact', 'king')
    contact.post({
        'contact_name': 'king',
        'alias': 'kong',
        'template': 'monstarrs'
    })
    r = Change.post()
    eq_(r.text, u'"Changes saved"')
    r = Change.get()
    eq_(len(r.json), 0, "No changes should be left")

    contact.patch({
        'service_notifications_enabled': False
    })
    r = Change.get()
    eq_(len(r.json), 1, "One change should have been made")

    cont_json = contact.get()
    eq_(cont_json['service_notifications_enabled'], False, "Service notifications should be disabled, we *did* ask for it")

    contact.delete()
    contact.verify_gone()
    template.delete()
    template.verify_gone()
    Change.post()

def test_limited_user_can_create_host():
    username = 'kavinsky'
    password = 'nightcall'

    user = Crud('user', username)
    user.post({
        'username': username,
        'password': password,
        'groups': ['admins', 'limited_view', 'limited_edit']
    }, cmp=False)

    contact = Crud('contact', username)
    contact.post({
        'contact_name': username,
        'alias': username
    })

    # we can't compare contacts since we're not allowed to
    # get them back, even though we just were able to create
    # them. sigh
    host = Crud('host', 'protovision')
    host.request_std_params['auth'] = (username, password)
    out = host.post({
        'host_name': 'protovision',
        'address': 'LV-426',
        'contacts': [username]
    }, cmp=False)
    eq_(out['host_name'], 'protovision', "Should've received host name 'protovision' but didn't")
    eq_(out['address'], 'LV-426', "Should've received address 'LV-426' but didn't")
    host.request_std_params['auth'] = ('monitor', 'monitor')
    host.delete()
    host.verify_gone()

    user.delete()
    user.verify_gone()

    contact.delete()
    contact.verify_gone()
    time.sleep(5)
